/***************************************************************************
                          rt_sensor.c  -  description
                             -------------------
    begin                : Sun Jun 11 2000
    copyright            : (C) 2000 by Joris Struyve
    email                : j.struyve@village.uunet.be
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


/* includes */

#include <rtl.h>
#include <rtl_fifo.h>
#include <rtl_time.h>
#include <time.h>
#include <rtl_sched.h>
#include <rtl_sync.h>
#include <pthread.h>
#include <unistd.h>
#include <rtl_debug.h>
#include <errno.h>

#include <k8000.h>
#include "tempd.h"

/* global vars */
int ReadPulseLength(int io_channel,int pulscount);

/* structs an defines */
#define SCAN_TIMEOUT 3000000000 /* timeout is 3 seconds */

int fifo_handler(unsigned int fifo)
{
	scan_request request;
	scan_response response;
	int err;
	
	while (  (err = rtf_get(REQUEST_FIFO_ID, &request, sizeof(request))) == sizeof(scan_request)   )
	{
		/*rtl_printf("Extracting data (%d bytes long)\n",err);
		rtl_printf("     iochannel   = %d\n",request.io_channel);
		rtl_printf("     noof_pulses = %d\n",request.noof_pulses);*/
	
		/* read the length */
		response.value = ReadPulseLength(request.io_channel,request.noof_pulses);
	
		/* complete the response struct */
		response.io_channel = request.io_channel;
	
		/* log something */
		rtl_printf("rt_tempd : Pulse up length is %d\n", response.value);
	
		/* put the response on the fifo */
		rtf_put (RESPONSE_FIFO_ID, &response, sizeof(scan_response));
	}
	return 0;
}

int ReadPulseLength(int io_channel,int pulscount)
{

	int pulsefound=0;
	int flag_up=0;
	int tmp=1;
	
	hrtime_t down=0;
	hrtime_t up=0;
	hrtime_t cur_length=0;
	hrtime_t total=0;
	hrtime_t end_time=0;
	hrtime_t cur_time=0;

	end_time = gethrtime() + SCAN_TIMEOUT;
	
	ReadIOchannel(io_channel);
	tmp = IO[io_channel];
	while( pulscount>pulsefound )
	{
		/* make sure we don't loop forever */
		cur_time = gethrtime();
		if(end_time<cur_time)
		{
			SetIOchannel(2);
			return -1;
		}
			
		/* read the channel */
		ReadIOchannel(io_channel);

		/* if we have an event */
		if(tmp==0&&IO[io_channel]==1) /* change upwards */
		{
			/* register uptime */
			up = clock_gethrtime(CLOCK_UST);
			flag_up = 1;
		}
		else if (tmp==1&&IO[io_channel]==0&&flag_up) /*change downwards */
		{
			down = clock_gethrtime(CLOCK_UST);
			cur_length = down - up;
			total = total + cur_length;

			pulsefound++;
			flag_up = 0;
		}
		tmp = IO[io_channel];

	}
	/* calculate average length (after stripping of the max an min */
	ClearIOchannel(2);
	return ( ( ((int)(total))/(pulsefound))/1000 );
}

int init_module(void)
{
     int fifo_status;

     /* init the board */
    	SelectI2CprinterPort(1);
	I2CbusDelay = 1;
	ConfigIOchannelAsInput(16);

     /* Create the real time fifos */
	rtf_destroy(REQUEST_FIFO_ID);
	fifo_status = rtf_create(REQUEST_FIFO_ID, FIFO_SIZE);
	if (fifo_status) {
		rtl_printf("rt_sensor : Error request creating fifo, status=%d\n",fifo_status);
		return -1;
	}

	rtf_destroy(RESPONSE_FIFO_ID);
	fifo_status = rtf_create(RESPONSE_FIFO_ID, FIFO_SIZE);
	if (fifo_status) {
		rtl_printf("rt_sensor : Error response creating fifo, status=%d\n",fifo_status);
		return -1;
	}
			
	/* attach the fifo handler to the request fifo */
	if(rtf_create_handler(REQUEST_FIFO_ID, &fifo_handler))
	{
		rtl_printf("Failed to create fifo handler (quit)\n");
		return -1;
	}
	rtl_printf("rt_tempd : module started (channel 1 should be up)\n");
	
	/* light up channel 1 to indicate we're running */
	SetIOchannel(1);
	return 0;
}

void cleanup_module(void)
{
	rtl_printf ("rt_tempd : Removing module on CPU %d\n", rtl_getcpuid());
	ClearIOchannel(1);
	ClearIOchannel(2);
	/* destroy the fifos */
	rtf_destroy(REQUEST_FIFO_ID);
	rtf_destroy(RESPONSE_FIFO_ID);
}

