#include <k8000.h>
#include "tempd.h"

#include <sys/time.h>
#include <sys/types.h>
#include <signal.h>
#include <stdio.h>
#include <pthread.h>
#include <unistd.h>
#include <errno.h>
#include <sys/stat.h>
#include <fcntl.h>


/* defines */
#define NOOF_SCANS 1

/* global  vars */
int noof_meters;
int fd_response_fifo;
int fd_request_fifo;

int main(int argc, char **argv)
{

	struct tm *tijd;
	time_t sec_tijd;
	long ref_time;
	int i;
	char scantime[20];
	char *err;
	meter *meters=NULL;
	pthread_t thread;

	signal(SIGTERM,Signal);
	signal(SIGINT,Signal);
	
	if(InitMeters(&meters,"js/js@lyra"))
		exit(0);

	/* Connect to the database */
	if(ConnectToOracleByString("js/js@lyra",&err))
	{
		printf("Error connecting : %s\n",err);
		exit(1);
	}
	
	/* open the response queue */
	if ((fd_response_fifo = open(RESPONSE_FIFO, O_RDONLY)) < 0)
	{
		printf("Error opening %s : %s\n",RESPONSE_FIFO,strerror(errno));
		exit(1);
	}

	/* open the request queue */
	if ((fd_request_fifo = open(REQUEST_FIFO, O_WRONLY)) < 0)
	{
		printf("Error opening %s : %s\n",REQUEST_FIFO,strerror(errno));
		exit(1);
	}

	/* run forever */
	for(;;)
	{
		/* get the current time and calculate the reftime in seconds */
		time(&sec_tijd);
		tijd = localtime(&sec_tijd);
		ref_time = tijd->tm_sec + (tijd->tm_min * 60) + (tijd->tm_hour * 60 * 60);

		/* for every meter */
		for(i=0;i<noof_meters;i++)
		{
			/* is the interval for this meter reached ? */
			if( !( ref_time%((meters+i)->interval) ))
			{
				printf("Interval for %s elapsed at %s",(meters+i)->code,asctime(tijd));
				strftime(scantime,20,"%d/%m/%G %H:%M:%S",tijd);
				if(ScanMeter(meters+i,scantime))
					printf("Error scanning meter %s\n",(meters+i)->code);
			}
		}
		/* have a rest */
		sleep(1);
	}
	
	/* the code never get's here, but it makes me feel good */
	DisconnectFromOracle();
}

int ScanMeter(meter *mtr,char* scantime)
{
	float cal_value;
	int i,retval;
	char *err=NULL;
	scan_request request;
	scan_response response;
	struct timeval tv;
	int ret_err;
	fd_set rfds;

	tv.tv_sec = 1;
	tv.tv_usec = 0;

	
	/* Send scan request on the realtime fifo */
	request.io_channel = mtr->io_channel;
	request.noof_pulses = 12;
	
	if (  (retval=write(fd_request_fifo,&request,sizeof(scan_request) ))!= sizeof(scan_request) )
	{
		printf("only %d bytes written : %s\n",retval,strerror(errno));
		printf("fifo %d\n",fd_request_fifo);
		exit(1);
	}
	

	/* Watch te response rtfifo */
	FD_ZERO(&rfds);
	FD_SET(fd_response_fifo, &rfds);
	retval = select(FD_SETSIZE,&rfds,NULL,NULL,NULL);
	if (retval > 0)
	{
		if ((ret_err = read(fd_response_fifo, &response, sizeof(response))) == sizeof(response))
		{
			/* calculate the temperature  temp = a* raw_value + b */
			cal_value = mtr->a * response.value + mtr->b;
			printf("%d ===> %f C\n",response.value,cal_value);
			
			/* insert the value in oracle */
			if(InsertMeterValue(mtr->code,response.value,cal_value,scantime,&err))
			{
				printf("Error writing value for %s : %s\n",mtr->code,err);
				return 1;
			}

		}
		else
			printf("Wrong noof bytes (%d) on pipe\n",ret_err);
	}
	return 0;
}

int InitMeters(meter **meters,char *connectstr)
{
	int noof_points;
	int i,j;
	char *err=NULL;
	calibration_point *cal_points=NULL;
	/* vars to calc the regression */
	float Ex,Ey,Ex2,Exy;
	
	printf("Checking connectstring............");
	if(ConnectToOracleByString(connectstr,&err))
	{
		printf("Failed!\n");
		printf("       ==> %s\n",err);
		return 1;
	}
	printf(" OK.\n");
	
	/* fill the meters struct array */
	printf("Gettin' the meter definitions.....");
	if(GetMeters(meters,&noof_points,&err))
	{
		printf("Failed!\n");
		printf("       ==> %s\n",err);
		return 1;
	}
	noof_meters = noof_points;
	printf(" OK.(%d found)\n",noof_points);
	
	/* calculate y = ax + b for every meter */
	for(j=0;j<noof_meters;j++)
	{
		/*printf("meter %d (at %p)\n",i,((*meters)+j));
		printf("   code = %s\n",((*meters)+j)->code);
		printf("   io   = %d\n",((*meters)+j)->io_channel);
		printf("   inter= %d\n",((*meters)+j)->interval);*/
		
		printf("     Regression curve for %s :...",((*meters)+j)->code);
		
		if(GetCalibrationPoints(((*meters)+j)->code,&cal_points,&noof_points,&err))
		{
			printf("Failed!\n");
			printf("       ==> %s\n",err);
			return 1;
		}

		/*printf("Noof points returned for %s = %d\n",(*meters)->code,noof_points);*/
		Ex = Ey = Ex2 = Exy = 0;
		for(i=0;i<noof_points;i++)
		{
			/*printf("Point %d (at %p)\n",i,(cal_points+i));
			printf("   xval = %d\n",(cal_points+i)->x_val);
			printf("   yval = %d\n",(cal_points+i)->y_val);*/
			Ex = Ex + (cal_points+i)->x_val;
			Ey = Ey + (cal_points+i)->y_val;
			Ex2 = Ex2 + ((cal_points+i)->x_val * (cal_points+i)->x_val);
			Exy = Exy + ((cal_points+i)->x_val * (cal_points+i)->y_val);
		}
		((*meters)+j)->a = (Exy - (( Ex * Ey )/noof_points))/( Ex2 - ( (Ex * Ex )/noof_points));
		((*meters)+j)->b = (Ey/noof_points)-(( ((*meters)+j)->a )*(Ex/noof_points));
		
		printf(" Temp = %f * pulse width + %f\n",((*meters)+j)->a,((*meters)+j)->b);
	}	
	DisconnectFromOracle();
	return 0;

}


void Signal(int sig)
{
	/*int loop1,i,delay=800000;*/
	printf("\nSignal %d received\n",sig);
	DisconnectFromOracle();
	exit(0);
}

